@@ -5,14 +5,40 @@ from collections import OrderedDict
 from sql import Literal
 from sql.conditionals import Coalesce
 
-from trytond.model import (fields, ModelView, ModelSQL, MatchMixin,
-    sequence_ordered)
-from trytond.pyson import Eval, If, Bool
+from trytond.i18n import gettext
+from trytond.model import (
+    MatchMixin, ModelSQL, ModelView, Workflow, fields, sequence_ordered)
+from trytond.modules.account.exceptions import ClosePeriodError
+from trytond.modules.company.model import CompanyValueMixin
 from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Bool, Eval, Id, If
 from trytond.tools import grouped_slice
 from trytond.transaction import Transaction
 
 
+class Configuration(metaclass=PoolMeta):
+    __name__ = 'account.configuration'
+
+    default_customer_payment_term = fields.MultiValue(
+        fields.Many2One(
+            'account.invoice.payment_term', "Default Customer Payment Term"))
+
+    @classmethod
+    def multivalue_model(cls, field):
+        pool = Pool()
+        if field in 'default_customer_payment_term':
+            return pool.get('account.configuration.default_payment_term')
+        return super().multivalue_model(field)
+
+
+class ConfigurationDefaultPaymentTerm(ModelSQL, CompanyValueMixin):
+    "Account Configuration Default Payment Term"
+    __name__ = 'account.configuration.default_payment_term'
+
+    default_customer_payment_term = fields.Many2One(
+        'account.invoice.payment_term', "Default Customer Payment Term")
+
+
 class FiscalYear(metaclass=PoolMeta):
     __name__ = 'account.fiscalyear'
     invoice_sequences = fields.One2Many(
@@ -20,8 +46,7 @@ class FiscalYear(metaclass=PoolMeta):
         "Invoice Sequences",
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
 
     @classmethod
     def __register__(cls, module_name):
@@ -111,6 +136,27 @@ class Period(metaclass=PoolMeta):
             table.drop_column('in_invoice_sequence')
             table.drop_column('in_credit_note_sequence')
 
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('close')
+    def close(cls, periods):
+        pool = Pool()
+        Invoice = pool.get('account.invoice')
+        company_ids = list({p.company.id for p in periods})
+        invoices = Invoice.search([
+                ('company', 'in', company_ids),
+                ('state', '=', 'posted'),
+                ('move', '=', None),
+                ])
+        if invoices:
+            names = ', '.join(i.rec_name for i in invoices[:5])
+            if len(invoices) > 5:
+                names += '...'
+            raise ClosePeriodError(
+                gettext('account_invoice.msg_close_period_non_posted_invoices',
+                    invoices=names))
+        super().close(periods)
+
 
 class InvoiceSequence(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
     'Invoice Sequence'
@@ -120,54 +166,40 @@ class InvoiceSequence(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         'account.fiscalyear', "Fiscal Year", required=True, ondelete='CASCADE',
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
     period = fields.Many2One('account.period', 'Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('type', '=', 'standard'),
-            ],
-        depends=['fiscalyear'])
+            ])
     in_invoice_sequence = fields.Many2One('ir.sequence.strict',
         'Supplier Invoice Sequence', required=True,
         domain=[
-            ('code', '=', 'account.invoice'),
-            ['OR',
-                ('company', '=', Eval('company')),
-                ('company', '=', None),
-                ],
-            ],
-        depends=['company'])
+            ('sequence_type', '=',
+                Id('account_invoice', 'sequence_type_account_invoice')),
+            ('company', '=', Eval('company')),
+            ])
     in_credit_note_sequence = fields.Many2One('ir.sequence.strict',
         'Supplier Credit Note Sequence', required=True,
         domain=[
-            ('code', '=', 'account.invoice'),
-            ['OR',
-                ('company', '=', Eval('company')),
-                ('company', '=', None),
-                ],
-            ],
-        depends=['company'])
+            ('sequence_type', '=',
+                Id('account_invoice', 'sequence_type_account_invoice')),
+            ('company', '=', Eval('company')),
+            ])
     out_invoice_sequence = fields.Many2One('ir.sequence.strict',
         'Customer Invoice Sequence', required=True,
         domain=[
-            ('code', '=', 'account.invoice'),
-            ['OR',
-                ('company', '=', Eval('company')),
-                ('company', '=', None),
-                ],
-            ],
-        depends=['company'])
+            ('sequence_type', '=',
+                Id('account_invoice', 'sequence_type_account_invoice')),
+            ('company', '=', Eval('company')),
+            ])
     out_credit_note_sequence = fields.Many2One('ir.sequence.strict',
         'Customer Credit Note Sequence', required=True,
         domain=[
-            ('code', '=', 'account.invoice'),
-            ['OR',
-                ('company', '=', Eval('company')),
-                ('company', '=', None),
-                ],
-            ],
-        depends=['company'])
+            ('sequence_type', '=',
+                Id('account_invoice', 'sequence_type_account_invoice')),
+            ('company', '=', Eval('company')),
+            ])
 
     @classmethod
     def __setup__(cls):
@@ -201,8 +233,7 @@ class MoveLine(metaclass=PoolMeta):
                 ],
             states={
                 'invisible': Bool(Eval('reconciliation')),
-                },
-            depends=['account', 'party', 'reconciliation']),
+                }),
         'get_invoice_payment',
         setter='set_invoice_payment',
         searcher='search_invoice_payment')
@@ -215,11 +246,26 @@ class MoveLine(metaclass=PoolMeta):
         super(MoveLine, cls).__setup__()
         cls._check_modify_exclude.add('invoice_payment')
 
+    @classmethod
+    def _view_reconciliation_muted(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        muted = super()._view_reconciliation_muted()
+        muted.add(ModelData.get_id(
+            'account_invoice', 'move_line_view_list_to_pay'))
+        return muted
+
     @classmethod
     def _get_origin(cls):
         return super()._get_origin() + [
             'account.invoice.line', 'account.invoice.tax']
 
+    @classmethod
+    def copy(cls, lines, default=None):
+        default = {} if default is None else default.copy()
+        default.setdefault('invoice_payments', None)
+        return super().copy(lines, default=default)
+
     @classmethod
     def get_invoice_payment(cls, lines, name):
         pool = Pool()
@@ -271,8 +317,10 @@ def _invoices_to_process(reconciliations):
 
     invoices = set()
     for sub_ids in grouped_slice(move_ids):
-        invoices.update(Invoice.search([
-                    ('move', 'in', list(sub_ids)),
+        sub_ids = list(sub_ids)
+        invoices.update(Invoice.search(['OR',
+                    ('move', 'in', sub_ids),
+                    ('additional_moves', 'in', sub_ids),
                     ]))
     if others:
         invoices.update(_invoices_to_process(Reconciliation.browse(others)))
@@ -286,17 +334,25 @@ class Reconciliation(metaclass=PoolMeta):
     @classmethod
     def create(cls, vlist):
         Invoice = Pool().get('account.invoice')
+        transaction = Transaction()
+        context = transaction.context
         reconciliations = super(Reconciliation, cls).create(vlist)
-        Invoice.process(list(_invoices_to_process(reconciliations)))
+        with transaction.set_context(
+                queue_batch=context.get('queue_batch', True)):
+            Invoice.__queue__.process(
+                list(_invoices_to_process(reconciliations)))
         return reconciliations
 
     @classmethod
     def delete(cls, reconciliations):
         Invoice = Pool().get('account.invoice')
-
+        transaction = Transaction()
+        context = transaction.context
         invoices_to_process = _invoices_to_process(reconciliations)
         super(Reconciliation, cls).delete(reconciliations)
-        Invoice.process(list(invoices_to_process))
+        with transaction.set_context(
+                queue_batch=context.get('queue_batch', True)):
+            Invoice.__queue__.process(list(invoices_to_process))
 
 
 class RenewFiscalYear(metaclass=PoolMeta):
@@ -364,3 +420,23 @@ class RenewFiscalYear(metaclass=PoolMeta):
         if to_write:
             InvoiceSequence.write(*to_write)
         return fiscalyear
+
+
+class RescheduleLines(metaclass=PoolMeta):
+    __name__ = 'account.move.line.reschedule'
+
+    @classmethod
+    def reschedule_lines(cls, lines, journal, terms):
+        pool = Pool()
+        Invoice = pool.get('account.invoice')
+        move, balance_line = super().reschedule_lines(lines, journal, terms)
+
+        move_ids = list({l.move.id for l in lines})
+        invoices = Invoice.search(['OR',
+                ('move', 'in', move_ids),
+                ('additional_moves', 'in', move_ids),
+                ])
+        Invoice.write(invoices, {
+                'additional_moves': [('add', [move.id])],
+                })
+        return move, balance_line
