@@ -2,12 +2,11 @@
 # this repository contains the full copyright notices and license terms.
 from trytond import backend
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, ValueMixin, fields
+from trytond.model import ModelSQL, ModelView, ValueMixin, fields
+from trytond.modules.party.exceptions import EraseError
 from trytond.pool import Pool, PoolMeta
 from trytond.tools.multivalue import migrate_property
 
-from trytond.modules.party.exceptions import EraseError
-
 customer_payment_term = fields.Many2One(
     'account.invoice.payment_term', "Customer Payment Term")
 supplier_payment_term = fields.Many2One(
@@ -31,7 +30,7 @@ class ContactMechanism(metaclass=PoolMeta):
         return super(ContactMechanism, cls).usages(_fields=_fields)
 
 
-class Party(ModelSQL, ModelView):
+class Party(metaclass=PoolMeta):
     __name__ = 'party.party'
     customer_payment_term = fields.MultiValue(customer_payment_term)
     supplier_payment_term = fields.MultiValue(supplier_payment_term)
@@ -45,6 +44,15 @@ class Party(ModelSQL, ModelView):
             return pool.get('party.party.payment_term')
         return super(Party, cls).multivalue_model(field)
 
+    @classmethod
+    def default_customer_payment_term(cls, **pattern):
+        pool = Pool()
+        Configuration = pool.get('account.configuration')
+        config = Configuration(1)
+        payment_term = config.get_multivalue(
+            'default_customer_payment_term', **pattern)
+        return payment_term.id if payment_term else None
+
 
 class PartyPaymentTerm(ModelSQL, ValueMixin):
     "Party Payment Term"
