@@ -1,21 +1,22 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
+
 from dateutil.relativedelta import relativedelta
+from sql import Column
 
-from sql import Column, Null
-
+from trytond import backend
+from trytond.config import config
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, sequence_ordered)
-from trytond import backend
+    DeactivableMixin, ModelSQL, ModelView, fields, sequence_ordered)
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
-from trytond.pool import Pool
-from trytond.wizard import Wizard, StateView, Button
-from trytond.config import config
+from trytond.wizard import Button, StateView, Wizard
 
-from .exceptions import PaymentTermValidationError, PaymentTermComputeError
+from .exceptions import PaymentTermComputeError, PaymentTermValidationError
 
 
 class PaymentTerm(DeactivableMixin, ModelSQL, ModelView):
@@ -32,19 +33,22 @@ class PaymentTerm(DeactivableMixin, ModelSQL, ModelView):
         cls._order.insert(0, ('name', 'ASC'))
 
     @classmethod
-    def validate(cls, terms):
-        super(PaymentTerm, cls).validate(terms)
+    def validate_fields(cls, terms, field_names):
+        super().validate_fields(terms, field_names)
+        cls.check_remainder(terms, field_names)
+
+    @classmethod
+    def check_remainder(cls, terms, field_names=None):
+        if field_names and 'lines' not in field_names:
+            return
         for term in terms:
-            term.check_remainder()
+            if not term.lines or not term.lines[-1].type == 'remainder':
+                raise PaymentTermValidationError(gettext(
+                        'account_invoice'
+                        '.msg_payment_term_missing_last_remainder',
+                        payment_term=term.rec_name))
 
-    def check_remainder(self):
-        if not self.lines or not self.lines[-1].type == 'remainder':
-            raise PaymentTermValidationError(
-                gettext('account_invoice'
-                    '.msg_payment_term_missing_last_remainder',
-                    payment_term=self.rec_name))
-
-    def compute(self, amount, currency, date=None):
+    def compute(self, amount, currency, date):
         """Calculate payment terms and return a list of tuples
         with (date, amount) for each payment term line.
 
@@ -54,12 +58,8 @@ class PaymentTerm(DeactivableMixin, ModelSQL, ModelView):
         """
         # TODO implement business_days
         # http://pypi.python.org/pypi/BusinessHours/
-        Date = Pool().get('ir.date')
-
         sign = 1 if amount >= Decimal('0.0') else -1
         res = []
-        if date is None:
-            date = Date.today()
         remainder = amount
         for line in self.lines:
             value = line.get_value(remainder, amount, currency)
@@ -99,27 +99,31 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
         states={
             'invisible': ~Eval('type').in_(['percent', 'percent_on_total']),
             'required': Eval('type').in_(['percent', 'percent_on_total']),
-            }, depends=['type'])
+            })
     divisor = fields.Numeric('Divisor', digits=(10, 14),
         states={
             'invisible': ~Eval('type').in_(['percent', 'percent_on_total']),
             'required': Eval('type').in_(['percent', 'percent_on_total']),
-            }, depends=['type'])
-    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
+            })
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency',
         states={
             'invisible': Eval('type') != 'fixed',
             'required': Eval('type') == 'fixed',
-            }, depends=['type', 'currency_digits'])
+            })
     currency = fields.Many2One('currency.currency', 'Currency',
         states={
             'invisible': Eval('type') != 'fixed',
             'required': Eval('type') == 'fixed',
-            }, depends=['type'])
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
+            })
     relativedeltas = fields.One2Many(
         'account.invoice.payment_term.line.delta', 'line', 'Deltas')
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('payment')
+
     @classmethod
     def __register__(cls, module_name):
         sql_table = cls.__table__()
@@ -134,10 +138,6 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
                     values=[sql_table.percentage / 100]))
             table.drop_column('percentage')
 
-    @staticmethod
-    def default_currency_digits():
-        return 2
-
     @staticmethod
     def default_type():
         return 'remainder'
@@ -173,12 +173,6 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
             self.ratio = self.round(1 / self.divisor,
                 self.__class__.ratio.digits[1])
 
-    @fields.depends('currency')
-    def on_change_with_currency_digits(self, name=None):
-        if self.currency:
-            return self.currency.digits
-        return 2
-
     def get_date(self, date):
         for relativedelta_ in self.relativedeltas:
             date += relativedelta_.get()
@@ -203,15 +197,16 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
         return Decimal(number).quantize(quantize)
 
     @classmethod
-    def validate(cls, lines):
-        super(PaymentTermLine, cls).validate(lines)
-        cls.check_ratio_and_divisor(lines)
+    def validate_fields(cls, lines, field_names):
+        super().validate_fields(lines, field_names)
+        cls.check_ratio_and_divisor(lines, field_names)
 
     @classmethod
-    def check_ratio_and_divisor(cls, lines):
+    def check_ratio_and_divisor(cls, lines, field_names=None):
         "Check consistency between ratio and divisor"
-        # Use a copy because on_change will change the records
-        for line in cls.browse(lines):
+        if field_names and not (field_names & {'type', 'ratio', 'divisor'}):
+            return
+        for line in lines:
             if line.type not in ('percent', 'percent_on_total'):
                 continue
             if line.ratio is None or line.divisor is None:
@@ -245,6 +240,11 @@ class PaymentTermLineRelativeDelta(sequence_ordered(), ModelSQL, ModelView):
     weeks = fields.Integer('Number of Weeks', required=True)
     days = fields.Integer('Number of Days', required=True)
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('line')
+
     @classmethod
     def __register__(cls, module_name):
         transaction = Transaction()
@@ -257,6 +257,7 @@ class PaymentTermLineRelativeDelta(sequence_ordered(), ModelSQL, ModelView):
         line = Line.__table__()
         month = Month.__table__()
         day = Day.__table__()
+        table_h = cls.__table_handler__(module_name)
 
         # Migration from 4.0: rename long table
         old_model_name = 'account.invoice.payment_term.line.relativedelta'
@@ -267,21 +268,13 @@ class PaymentTermLineRelativeDelta(sequence_ordered(), ModelSQL, ModelView):
 
         # Migration from 5.0: use ir.calendar
         migrate_calendar = False
-        if backend.TableHandler.table_exist(cls._table):
-            cursor.execute(*sql_table.select(
-                    sql_table.month, sql_table.weekday,
-                    where=(sql_table.month != Null)
-                    | (sql_table.weekday != Null),
-                    limit=1))
-            try:
-                row, = cursor.fetchall()
-                migrate_calendar = any(isinstance(v, str) for v in row)
-            except ValueError:
-                # As we cannot know the column type
-                # we migrate any way as no data need to be migrated
-                migrate_calendar = True
+        if (backend.TableHandler.table_exist(cls._table)
+                and table_h.column_exist('month')
+                and table_h.column_exist('weekday')):
+            migrate_calendar = (
+                table_h.column_is_type('month', 'VARCHAR')
+                or table_h.column_is_type('weekday', 'VARCHAR'))
             if migrate_calendar:
-                table_h = cls.__table_handler__(module_name)
                 table_h.column_rename('month', '_temp_month')
                 table_h.column_rename('weekday', '_temp_weekday')
 
@@ -364,10 +357,9 @@ class TestPaymentTermView(ModelView):
     payment_term = fields.Many2One('account.invoice.payment_term',
         'Payment Term', required=True)
     date = fields.Date('Date')
-    amount = fields.Numeric('Amount', required=True,
-        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', required=True)
     currency = fields.Many2One('currency.currency', 'Currency', required=True)
-    currency_digits = fields.Integer('Currency Digits')
     result = fields.One2Many('account.invoice.payment_term.test.result',
         None, 'Result', readonly=True)
 
@@ -379,12 +371,6 @@ class TestPaymentTermView(ModelView):
         if company:
             return Company(company).currency.id
 
-    @fields.depends('currency')
-    def on_change_with_currency_digits(self):
-        if self.currency:
-            return self.currency.digits
-        return 2
-
     @fields.depends('payment_term', 'date', 'amount', 'currency', 'result')
     def on_change_with_result(self):
         pool = Pool()
@@ -396,8 +382,7 @@ class TestPaymentTermView(ModelView):
                 result.append(Result(
                         date=date,
                         amount=amount,
-                        currency=self.currency,
-                        currency_digits=self.currency.digits))
+                        currency=self.currency))
         self.result = result
         return self._changed_values.get('result', [])
 
@@ -406,7 +391,6 @@ class TestPaymentTermViewResult(ModelView):
     'Test Payment Term'
     __name__ = 'account.invoice.payment_term.test.result'
     date = fields.Date('Date', readonly=True)
-    amount = fields.Numeric('Amount', readonly=True,
-        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', readonly=True)
     currency = fields.Many2One('currency.currency', "Currency")
-    currency_digits = fields.Integer('Currency Digits')
